@@ -20,7 +20,7 @@
 #include <stdbool.h>
 
 #ifndef CRIPPLED_LIBC
-const char *
+char *
 asinfo_from_rr(const char *rrtype, const char *rdata, char **asn, char **cidr);
 #endif
 
