@@ -65,6 +65,7 @@ static void qdesc_debug(const char *, qdesc_ct);
 static void qparam_debug(const char *, qparam_ct);
 static __attribute__((noreturn)) void usage(const char *, ...);
 static bool parse_long(const char *, long *);
+static void set_timeout(const char *, const char *);
 static const char *qparam_ready(qparam_t);
 static const char *qparam_option(int, const char *, qparam_t);
 static verb_ct find_verb(const char *);
@@ -91,17 +92,25 @@ static const char * const conf_files[] = {
 	NULL
 };
 
+const struct presenter pres_text_lookup = { present_text_lookup, true };
+const struct presenter pres_json_lookup = { present_json_lookup, true };
+const struct presenter pres_csv_lookup = { present_csv_lookup, true };
+const struct presenter pres_minimal_lookup = { present_minimal_lookup, false };
+const struct presenter pres_text_summarize = { present_text_summarize, true };
+const struct presenter pres_json_summarize = { present_json_summarize, true };
+const struct presenter pres_csv_summarize = { present_csv_summarize, true };
+
 const struct verb verbs[] = {
 	/* note: element [0] of this array is the DEFAULT_VERB. */
 	{ "lookup", "/lookup", lookup_ok,
-	  present_text_lookup,
-	  present_json_lookup,
-	  present_csv_lookup,
-	  present_minimal_lookup },
+	  &pres_text_lookup,
+	  &pres_json_lookup,
+	  &pres_csv_lookup,
+	  &pres_minimal_lookup },
 	{ "summarize", "/summarize", summarize_ok,
-	  present_text_summarize,
-	  present_json_summarize,
-	  present_csv_summarize,
+	  &pres_text_summarize,
+	  &pres_json_summarize,
+	  &pres_csv_summarize,
 	  NULL },
 	{ NULL, NULL, NULL, NULL, NULL, NULL, NULL }
 };
@@ -127,23 +136,27 @@ main(int argc, char *argv[]) {
 	/* global dynamic initialization. */
 	ideal_buffer = 4 * (size_t) sysconf(_SC_PAGESIZE);
 	gettimeofday(&startup_time, NULL);
+
 	if ((program_name = strrchr(argv[0], '/')) == NULL)
 		program_name = argv[0];
 	else
 		program_name++;
-	value = getenv(env_time_fmt);
-	if (value != NULL && strcasecmp(value, "iso") == 0)
+
+	if ((value = getenv(env_time_fmt)) != NULL &&
+	    strcasecmp(value, "iso") == 0)
 		iso8601 = true;
 
-	value = getenv(env_config_file);
-	if (value != NULL)
+	if ((value = getenv(env_config_file)) != NULL)
 		config_file = strdup(value);
 
+	if ((value = getenv(env_timeout)) != NULL)
+		set_timeout(value, env_timeout);
+
 	pverb = &verbs[DEFAULT_VERB];
 
 	/* process the command line options. */
 	while ((ch = getopt(argc, argv,
-			    "D:R:r:N:n:i:M:u:p:t:b:k:J:V:T:0:"
+			    "C:D:R:r:N:n:i:M:u:p:t:b:k:J:V:T:0:o:"
 			    "adfhIjmqSsUv468" QPARAM_GETOPT))
 	       != -1)
 	{
@@ -174,6 +187,9 @@ main(int argc, char *argv[]) {
 		case 'a':
 			asinfo_lookup = true;
 			break;
+		case 'C':
+			cookie_file = strdup(optarg);
+			break;
 		case 'D':
 			asinfo_domain = optarg;
 			break;
@@ -321,6 +337,9 @@ main(int argc, char *argv[]) {
 			    (max_count <= 0))
 				usage("-M must be positive");
 			break;
+		case 'o':
+			set_timeout(optarg, "-o");
+			break;
 		case 'u':
 			picked_system = strdup(optarg);
 			break;
@@ -414,6 +433,8 @@ main(int argc, char *argv[]) {
 					transforms |= TRANS_DATEFIX;
 				else if (strcasecmp(token, "chomp") == 0)
 					transforms |= TRANS_CHOMP;
+				else if (strcasecmp(token, "qdetail") == 0)
+					transforms |= TRANS_QDETAIL;
 				else {
 					usage("unrecognized transform in -T");
 				}
@@ -473,9 +494,8 @@ main(int argc, char *argv[]) {
 		usage("the -a option requires a modern functional C library.");
 #else
 		if (!asinfo_domain_exists(asinfo_domain)) {
-			fprintf(stderr,
-				"%s: ASINFO domain (%s) does not exist.\n",
-				program_name, asinfo_domain);
+			my_logf("ASINFO domain (%s) does not exist",
+				asinfo_domain);
 			my_exit(1);
 		}
 #endif
@@ -530,10 +550,25 @@ main(int argc, char *argv[]) {
 			my_panic(true, "asprintf");
 		usage(errmsg);
 	}
+	if (presentation != pres_json && (transforms & TRANS_QDETAIL) != 0)
+		usage("'-T qdetail' currently requires '-j' or '-p json'");
 
 	/* get to final readiness; in particular, get psys set. */
-	if (sorting != no_sort)
+	if (sorting != no_sort) {
+		if (!presenter->sortable) {
+			char *errmsg = NULL;
+			int x = asprintf(&errmsg,
+					 "that presentation format (%s) "
+					 "cannot be sorted",
+					 presentation_name);
+			if (x < 0)
+				my_panic(true, "asprintf");
+			usage(errmsg);
+		}
+		if ((transforms & TRANS_QDETAIL) != 0)
+			usage("\"-T qdetail\" is incompatible with sorting");
 		sort_ready();
+	}
 	if (config_file == NULL)
 		config_file = select_config();
 	if (picked_system != NULL) {
@@ -668,6 +703,7 @@ my_exit(int code) {
 
 	/* globals which may have been initialized, are to be freed. */
 	DESTROY(config_file);
+	DESTROY(cookie_file);
 	if (psys != NULL)
 		psys->destroy();
 
@@ -688,11 +724,8 @@ my_exit(int code) {
  */
 __attribute__((noreturn)) void
 my_panic(bool want_perror, const char *s) {
-	fprintf(stderr, "%s: ", program_name);
-	if (want_perror)
-		perror(s);
-	else
-		fprintf(stderr, "%s\n", s);
+	my_logf("panic (%s): ", s,
+		want_perror ? strerror(errno) : "generic");
 	my_exit(1);
 }
 
@@ -713,7 +746,7 @@ help(void) {
 	     "\t[-l QUERY-LIMIT] [-L OUTPUT-LIMIT]\n"
 	     "\t[-O OFFSET] [-M MAX_COUNT]\n"
 	     "\t[-A AFTER] [-B BEFORE]\n"
-	     "\t[-D ASINFO_DOMAIN] [-T (datefix|reverse|chomp)[,...] {\n"
+	     "\t[-D ASINFO_DOMAIN] [-T (datefix|reverse|chomp|qdetail)[,...] {\n"
 	     "\t\t-f |\n"
 	     "\t\t-J INPUTFILE |\n"
 	     "\t\t[-t RRTYPE[,...]] [-b BAILIWICK] {\n"
@@ -753,7 +786,7 @@ help(void) {
 	     "use -s to sort in ascending order, "
 	     "or -S for descending order.\n"
 	     "\t-s/-S can be repeated before several -k arguments.\n"
-	     "for -T, transforms are datefix, reverse, and chomp.\n"
+	     "for -T, transforms are datefix, reverse, chomp, and qdetail.\n"
 	     "use -U to turn off SSL certificate verification.\n"
 	     "use -v to show the program version.\n"
 	     "use -4 to force connecting to the server via IPv4.\n"
@@ -885,6 +918,16 @@ parse_long(const char *in, long *out) {
 	return true;
 }
 
+/* set_timeout -- ingest a setting for curl_timeout
+ *
+ * exits through usage() if the value is invalid.
+ */
+static void
+set_timeout(const char *value, const char *source) {
+	if (!parse_long(value, &curl_timeout) || (curl_timeout < 0))
+		usage("%s must be non-negative", source);
+}
+
 /* qparam_ready -- check and possibly adjust the contents of a qparam.
  */
 static const char *
@@ -1033,9 +1076,8 @@ do_batch(FILE *f, qparam_ct qpp) {
 				(sizeof "$options") - 1) == 0)
 		{
 			if ((msg = batch_options(command, &qp, qpp)) != NULL)
-				fprintf(stderr, "%s: warning: "
-					"batch option parse error: %s\n",
-					program_name, msg);
+				my_logf("warning: batch option parse error: %s",
+					msg);
 			continue;
 		}
 
@@ -1047,8 +1089,7 @@ do_batch(FILE *f, qparam_ct qpp) {
 		/* crack the batch line if possible. */
 		msg = batch_parse(command, &qd);
 		if (msg != NULL) {
-			fprintf(stderr, "%s: batch entry parse error: %s\n",
-				program_name, msg);
+			my_logf("batch entry parse error: %s", msg);
 		} else {
 			/* start one or more curl fetches based on this entry.
 			 */
@@ -1068,9 +1109,7 @@ do_batch(FILE *f, qparam_ct qpp) {
 			    batching != batch_verbose)
 			{
 				assert(query->message != NULL);
-				fprintf(stderr,
-					"%s: batch line status: %s (%s)\n",
-					program_name,
+				my_logf("batch line status: %s (%s)",
 					query->status, query->message);
 			}
 		}
@@ -1367,28 +1406,28 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 
 	/* ready player one. */
 	CREATE(query, sizeof(struct query));
-	query->descrip = makepath(qdp);
+	query->descr = makepath(qdp);
 	query->mode = qdp->mode;
-	query->params = *qpp;
+	query->qp = *qpp;
 	qpp = NULL;
 
 	/* define the fence. */
-	if (query->params.after != 0) {
-		if (query->params.complete) {
+	if (query->qp.after != 0) {
+		if (query->qp.complete) {
 			/* each db tuple must begin after the fence-start. */
-			fence.first_after = query->params.after;
+			fence.first_after = query->qp.after;
 		} else {
 			/* each db tuple must end after the fence-start. */
-			fence.last_after = query->params.after;
+			fence.last_after = query->qp.after;
 		}
 	}
-	if (query->params.before != 0) {
-		if (query->params.complete) {
+	if (query->qp.before != 0) {
+		if (query->qp.complete) {
 			/* each db tuple must end before the fence-end. */
-			fence.last_before = query->params.before;
+			fence.last_before = query->qp.before;
 		} else {
 			/* each db tuple must begin before the fence-end. */
-			fence.first_before = query->params.before;
+			fence.first_before = query->qp.before;
 		}
 	}
 
@@ -1399,8 +1438,7 @@ query_launcher(qdesc_ct qdp, qparam_ct qpp, writer_t writer) {
 		launch_fetch(query, path, &fence);
 		DESTROY(path);
 	} else if ((msg = rrtype_correctness(qdp->rrtype)) != NULL) {
-		fprintf(stderr, "%s: rrtype incorrect: %s\n",
-			program_name, msg);
+		my_logf("rrtype incorrect: %s", msg);
 		DESTROY(query);
 		return NULL;
 	} else {
@@ -1503,7 +1541,7 @@ rrtype_correctness(const char *input) {
  */
 static void
 launch_fetch(query_t query, const char *path, pdns_fence_ct fp) {
-	char *url = psys->url(path, NULL, &query->params, fp, false);
+	char *url = psys->url(path, NULL, &query->qp, fp, false);
 	if (url == NULL)
 		my_exit(1);
 
@@ -1525,7 +1563,7 @@ ruminate_json(int json_fd, qparam_ct qpp) {
 	writer = writer_init(qpp->output_limit, NULL, false);
 	CREATE(query, sizeof(struct query));
 	query->writer = writer;
-	query->params = *qpp;
+	query->qp = *qpp;
 	CREATE(fetch, sizeof(struct fetch));
 	fetch->query = query;
 	query->fetches = fetch;
