@@ -35,7 +35,9 @@
 static void present_text_line(const char *, const char *, const char *);
 static void present_csv_line(pdns_tuple_ct, const char *);
 static void present_minimal_thing(const char *thing);
-static json_t *annotate_json(pdns_tuple_ct, bool);
+static void present_json(pdns_tuple_ct, query_ct, bool);
+static json_t *annotate_json(pdns_tuple_ct, query_ct, bool);
+static json_t *annotation_json(query_ct query, json_t *annoRD);
 static json_t *annotate_one(json_t *, const char *, const char *, json_t *);
 #ifndef CRIPPLED_LIBC
 static json_t *annotate_asinfo(const char *, const char *);
@@ -46,7 +48,7 @@ static struct counted *countoff_r(const char *, int);
  */
 void
 present_text_lookup(pdns_tuple_ct tup,
-		    mode_e mode __attribute__ ((unused)),
+		    query_ct query __attribute__ ((unused)),
 		    writer_t writer __attribute__ ((unused)))
 {
 	bool pflag, ppflag;
@@ -168,7 +170,7 @@ present_text_line(const char *rrname, const char *rrtype, const char *rdata) {
  */
 void
 present_text_summarize(pdns_tuple_ct tup,
-		       mode_e mode __attribute__ ((unused)),
+		       query_ct query __attribute__ ((unused)),
 		       writer_t writer __attribute__ ((unused)))
 {
 	const char *prefix;
@@ -214,7 +216,7 @@ pprint_json(const char *buf, size_t len, FILE *outf) {
 
 	json_t *js = json_loadb(buf, len, 0, &error);
 	if (js == NULL) {
-		fprintf(stderr, "JSON parsing error %d:%d: %s %s\n",
+		my_logf("JSON parsing error %d:%d: %s %s",
 			error.line, error.column,
 			error.text, error.source);
 		return false;
@@ -231,27 +233,27 @@ pprint_json(const char *buf, size_t len, FILE *outf) {
  */
 void
 present_json_lookup(pdns_tuple_ct tup,
-		    mode_e mode __attribute__ ((unused)),
+		    query_ct query __attribute__ ((unused)),
 		    writer_t writer __attribute__ ((unused)))
 {
-	present_json(tup, true);
+	present_json(tup, query, true);
 }
 
 /* present_json_summarize -- render one DNSDB tuple as newline-separated JSON.
  */
 void
 present_json_summarize(pdns_tuple_ct tup,
-		       mode_e mode __attribute__ ((unused)),
+		       query_ct query __attribute__ ((unused)),
 		       writer_t writer __attribute__ ((unused)))
 {
-	present_json(tup, false);
+	present_json(tup, query, false);
 }
 
 /* present_json -- shared renderer for DNSDB JSON tuples (lookup and summarize)
  */
-void
-present_json(pdns_tuple_ct tup, bool rd) {
-	json_t *copy = annotate_json(tup, rd);
+static void
+present_json(pdns_tuple_ct tup, query_ct query, bool rd) {
+	json_t *copy = annotate_json(tup, query, rd);
 
 	if (copy != NULL) {
 		json_dumpf(copy, stdout, JSON_INDENT(0) | JSON_COMPACT);
@@ -266,7 +268,7 @@ present_json(pdns_tuple_ct tup, bool rd) {
 /* annotate_json -- create a temporary copy of a tuple; apply transforms.
  */
 static json_t *
-annotate_json(pdns_tuple_ct tup, bool rd) {
+annotate_json(pdns_tuple_ct tup, query_ct query, bool rd) {
 	json_t *annoRD = NULL, *annoTF = NULL, *annoTL = NULL,
 		*annoZF = NULL, *annoZL = NULL;
 
@@ -320,7 +322,7 @@ annotate_json(pdns_tuple_ct tup, bool rd) {
 	/* anything annotated? */
 	if ((annoZF != NULL && annoZL != NULL) ||
 	    (annoTF != NULL && annoTL != NULL) ||
-	    (transforms & (TRANS_REVERSE|TRANS_CHOMP)) != 0 ||
+	    (transforms & (TRANS_REVERSE|TRANS_CHOMP|TRANS_QDETAIL)) != 0 ||
 	    annoRD != NULL)
 	{
 		json_t *copy = json_deep_copy(tup->obj.cof_obj);
@@ -340,14 +342,61 @@ annotate_json(pdns_tuple_ct tup, bool rd) {
 		if ((transforms & (TRANS_REVERSE|TRANS_CHOMP)) != 0)
 			json_object_set_new_nocheck(copy, "rrname",
 						    json_string(tup->rrname));
-		if (annoRD != NULL)
-			json_object_set_new_nocheck(copy, "dnsdbq_rdata",
-						    annoRD);
+
+		if ((transforms & TRANS_QDETAIL) != 0 || annoRD != NULL) {
+			json_t *obj = annotation_json(query, annoRD);
+			if (obj != NULL)
+				json_object_set_new_nocheck(copy, "_dnsdbq",
+							    obj);
+		}
 		return copy;
 	}
 	return NULL;
 }
 
+static inline void
+instantiate_json(json_t **objptr) {
+	if (*objptr == NULL)
+		*objptr = json_object();
+}
+
+static json_t *
+annotation_json(query_ct query, json_t *annoRD) {
+	json_t *obj = NULL;
+
+	if (query != NULL && (transforms & TRANS_QDETAIL) != 0) {
+		instantiate_json(&obj);
+		if ((transforms & TRANS_QDETAIL) != 0)
+			json_object_set_new_nocheck(obj, "descr",
+						    json_string(query->descr));
+		if (query->qp.after != 0)
+			json_object_set_new_nocheck(obj, "after",
+				json_string_nocheck(
+					time_str(query->qp.after, iso8601)));
+		if (query->qp.before != 0)
+			json_object_set_new_nocheck(obj, "before",
+				json_string_nocheck(
+					time_str(query->qp.before, iso8601)));
+		if (query->qp.query_limit != -1)
+			json_object_set_new_nocheck(obj, "limit",
+				json_integer((json_int_t)
+					     query->qp.query_limit));
+		if (query->qp.offset != 0)
+			json_object_set_new_nocheck(obj, "offset",
+				json_integer((json_int_t)
+					     query->qp.offset));
+		json_object_set_new_nocheck(obj, "gravel",
+					    json_boolean(query->qp.gravel));
+		json_object_set_new_nocheck(obj, "complete",
+					    json_boolean(query->qp.complete));
+	}
+	if (annoRD != NULL) {
+		instantiate_json(&obj);
+		json_object_set_new_nocheck(obj, "anno", annoRD);
+	}
+	return obj;
+}
+
 static json_t *
 annotate_one(json_t *anno, const char *rdata, const char *name, json_t *obj) {
 	json_t *this = NULL;
@@ -398,7 +447,7 @@ annotate_asinfo(const char *rrtype, const char *rdata) {
  */
 void
 present_csv_lookup(pdns_tuple_ct tup,
-		   mode_e mode __attribute__ ((unused)),
+		   query_ct query __attribute__ ((unused)),
 		   writer_t writer)
 {
 	if (!writer->csv_headerp) {
@@ -490,16 +539,19 @@ present_csv_line(pdns_tuple_ct tup, const char *rdata) {
 	putchar('\n');
 }
 
-/* present_rdata_lookup -- render one DNSDB tuple as a "line" (MINIMAL)
+/* present_minimal_lookup -- render one DNSDB tuple as a "line"
  */
 void
 present_minimal_lookup(pdns_tuple_ct tup,
-		       mode_e mode __attribute__ ((unused)),
+		       query_ct query,
 		       writer_t writer __attribute__ ((unused)))
 {
+	/* here is why this presenter is incompatible with sorting. */
+	assert(query != NULL);
+
 	/* did this tuple come from a left hand or right hand query? */
 	bool left = true;
-	switch (mode) {
+	switch (query->mode) {
 	case no_mode:
 		abort();
 	case rrset_mode:
@@ -545,11 +597,11 @@ present_minimal_thing(const char *thing) {
 		puts(thing);
 }
 
-/* present_csv_summ -- render a summarize result as CSV.
+/* present_csv_summarize -- render a summarize result as CSV.
  */
 void
 present_csv_summarize(pdns_tuple_ct tup,
-		      mode_e mode __attribute__ ((unused)),
+		      query_ct query __attribute__ ((unused)),
 		      writer_t writer __attribute__ ((unused)))
 {
 	printf("time_first,time_last,zone_first,zone_last,"
@@ -589,14 +641,14 @@ tuple_make(pdns_tuple_t tup, const char *buf, size_t len) {
 	DEBUG(4, true, "[%d] '%-*.*s'\n", (int)len, (int)len, (int)len, buf);
 	tup->obj.main = json_loadb(buf, len, 0, &error);
 	if (tup->obj.main == NULL) {
-		fprintf(stderr, "%s: warning: json_loadb: %d:%d: %s %s\n",
-			program_name, error.line, error.column,
+		my_logf("warning: json_loadb: %d:%d: %s %s",
+			error.line, error.column,
 			error.text, error.source);
 		abort();
 	}
 	if (debug_level >= 4) {
 		char *pretty = json_dumps(tup->obj.main, JSON_INDENT(2));
-		fprintf(stderr, "debug: %s\n", pretty);
+		my_logf("%s", pretty);
 		free(pretty);
 	}
 
@@ -870,13 +922,13 @@ reverse(const char *src) {
 	return ret;
 }
 
-/* data_blob -- process one deblocked json blob as a counted string.
+/* pdns_blob -- process one deblocked json pdns blob as a counted string.
  *
- * presents, or outputs to POSIX sort(1), each blob and then frees it.
+ * presents, or outputs to POSIX sort(1), the blob, and then frees it.
  * returns number of tuples processed (for now, 1 or 0).
  */
 int
-data_blob(fetch_t fetch, size_t len) {
+pdns_blob(fetch_t fetch, size_t len) {
 	query_t query = fetch->query;
 	writer_t writer = query->writer;
 	struct pdns_tuple tup;
@@ -886,8 +938,7 @@ data_blob(fetch_t fetch, size_t len) {
 
 	msg = tuple_make(&tup, fetch->buf, len);
 	if (msg != NULL) {
-		fputs(msg, stderr);
-		fputc('\n', stderr);
+		my_logf("%s", msg);
 		goto more;
 	}
 
@@ -898,7 +949,7 @@ data_blob(fetch_t fetch, size_t len) {
 		}
 
 		if (tup.cond != NULL) {
-			DEBUG(5, true, "data_blob tup.cond = %s\n", tup.cond);
+			DEBUG(5, true, "pdns_blob tup.cond = %s\n", tup.cond);
 			/* if we goto next now, this line will not be counted.
 			 */
 			if (strcmp(tup.cond, "begin") == 0) {
@@ -920,9 +971,8 @@ data_blob(fetch_t fetch, size_t len) {
 			} else {
 				/* use sc_missing for an invalid cond value */
 				fetch->saf_cond = sc_missing;
-				fprintf(stderr,
-					"%s: Unknown value for \"cond\": %s\n",
-					program_name, tup.cond);
+				my_logf("Unknown value for \"cond\": %s",
+					tup.cond);
 			}
 		}
 
@@ -962,7 +1012,7 @@ data_blob(fetch_t fetch, size_t len) {
 		DEBUG(3, true, "dyn_rrname = '%s'\n", dyn_rrname);
 		DEBUG(3, true, "dyn_rdata = '%s'\n", dyn_rdata);
 		fprintf(writer->sort_stdin,
-			"%lu %lu %lu %lu %s %s %s %d %*.*s\n",
+			"%lu %lu %lu %lu %s %s %s %*.*s\n",
 			(unsigned long)first,
 			(unsigned long)last,
 			(unsigned long)(last - first),
@@ -970,9 +1020,8 @@ data_blob(fetch_t fetch, size_t len) {
 			or_else(dyn_rrname, "n/a"),
 			tup.rrtype,
 			or_else(dyn_rdata, "n/a"),
-			(int)query->mode,
 			(int)len, (int)len, fetch->buf);
-		DEBUG(2, true, "sort0: '%lu %lu %lu %lu %s %s %s %d %*.*s'\n",
+		DEBUG(2, true, "sort0: '%lu %lu %lu %lu %s %s %s %*.*s'\n",
 		      (unsigned long)first,
 		      (unsigned long)last,
 		      (unsigned long)(last - first),
@@ -980,12 +1029,12 @@ data_blob(fetch_t fetch, size_t len) {
 		      or_else(dyn_rrname, "n/a"),
 		      tup.rrtype,
 		      or_else(dyn_rdata, "n/a"),
-		      (int)query->mode,
 		      (int)len, (int)len, fetch->buf);
 		DESTROY(dyn_rrname);
 		DESTROY(dyn_rdata);
 	} else {
-		(*presenter)(&tup, query->mode, writer);
+		/* before the sort, we know the query that caused the tuple. */
+		(*presenter->output)(&tup, query, writer);
 	}
 
 	ret = 1;
@@ -1040,7 +1089,7 @@ pick_system(const char *name, const char *context) {
 	}
 
 	if (msg != NULL) {
-		fprintf(stderr, "%s (in %s)\n", msg, context);
+		my_logf("%s (in %s)\n", msg, context);
 		DESTROY(msg);
 		my_exit(1);
 	}
@@ -1080,8 +1129,7 @@ read_config(void) {
 	unsetenv("APIKEY");
 	f = popen(cmd, "r");
 	if (f == NULL) {
-		fprintf(stderr, "%s: [%s]: %s",
-			program_name, cmd, strerror(errno));
+		my_logf("[%s]: %s", cmd, strerror(errno));
 		DESTROY(cmd);
 		my_exit(1);
 	}
@@ -1097,17 +1145,14 @@ read_config(void) {
 
 		l++;
 		if (strchr(line, '\n') == NULL) {
-			fprintf(stderr, "%s: conf line #%d: too long\n",
-				program_name, l);
+			my_logf("conf line #%d: too long", l);
 			my_exit(1);
 		}
 		tok1 = strtok_r(line, "\040\012", &saveptr);
 		tok2 = strtok_r(NULL, "\040\012", &saveptr);
 		tok3 = strtok_r(NULL, "\040\012", &saveptr);
 		if (tok1 == NULL || tok2 == NULL) {
-			fprintf(stderr,
-				"%s: conf line #%d: malformed\n",
-				program_name, l);
+			my_logf("conf line #%d: malformed", l);
 			my_exit(1);
 		}
 		if (tok3 == NULL || *tok3 == '\0') {
@@ -1121,8 +1166,7 @@ read_config(void) {
 			if (strcmp(tok2, "system") == 0 && !psys_specified) {
 				pick_system(tok3, config_file);
 				if (psys == NULL) {
-					fprintf(stderr, "%s: unknown %s %s\n",
-						program_name,
+					my_logf("unknown %s %s\n",
 						DNSDBQ_SYSTEM,
 						tok3);
 					my_exit(1);
@@ -1138,16 +1182,16 @@ read_config(void) {
 			if (strcmp(tok2, "apikey") == 0) {
 				int ignored __attribute__((unused));
 				ignored = asprintf(&t, "[%zu]", strlen(tok3));
-			} else
+			} else {
 				t = strdup(tok3);
-			fprintf(stderr, "line #%d: sets %s|%s|%s\n",
-				l, tok1, tok2, t);
+			}
+			my_logf("line #%d: sets %s|%s|%s", l, tok1, tok2, t);
 			DESTROY(t);
 		}
 		if (strcmp(tok1, psys->name) == 0) {
 			msg = psys->setval(tok2, tok3);
 			if (msg != NULL) {
-				fprintf(stderr, "setval: %s\n", msg);
+				my_logf("setval: %s", msg);
 				my_exit(1);
 			}
 		}
