@@ -17,11 +17,13 @@
 #ifndef TIME_H_INCLUDED
 #define TIME_H_INCLUDED 1
 
+#include <sys/time.h>
 #include <sys/types.h>
 #include <stdbool.h>
 
 int time_cmp(u_long, u_long);
-const char * time_str(u_long, bool);
-int time_get(const char *src, u_long *dst);
+const char *time_str(u_long, bool);
+const char *timeval_str(const struct timeval *, bool);
+int time_get(const char *, u_long *);
 
 #endif /*TIME_H_INCLUDED*/
